#!/usr/bin/python

import os
import wx
import csnd6
import libs.inputPanel as inputPanel
import libs.metroPanel as metroPanel
import libs.seqPanel as seqPanel
import libs.dlyPanel as dlyPanel
import libs.outputPanel as outputPanel


#CSND CODE###########################################################
###################################################################
###################################################################
c = csnd6.Csound()    # create an instance of Csound

file = "csd/Spengger.csd"

c.Compile(file)     # Compile Orchestra from String

def updatePointer(csound):
	"""update gui"""
	#time bar
	kpointer = csound.GetChannel("kpointer")
	frame.matrixSeqP.gridPan.kpointer = kpointer
	#steps in gui
	ksteps = csound.GetChannel("ksteps_out")
	frame.matrixSeqP.gridPan.ksteps_out = ksteps
	#steps duration in gui
	kstepsdur = csound.GetChannel("ksteps_dur_out")
	frame.matrixSeqP.gridPan.ksteps_dur_out = kstepsdur
	return
	

#create csound performance thread
perfThread = csnd6.CsoundPerformanceThread(c)
# SET CALLBACKS HERE
perfThread.SetProcessCallback(updatePointer, c) 
#performance thread RUN
perfThread.Play()




#GUI CODE###########################################################
###################################################################
###################################################################
#Frame
class TopFrame(wx.Frame):
	"""Main frame"""
	#def __init__(self, *a, **k):
	def __init__(self, parent, title):
		#wx.Frame.__init__(self, *a, **k)
		#self.csound = k.pop('csound', None)
		#super(TopFrame, self).__init__(*a, **k)#super the subclass
		super(TopFrame, self).__init__(parent= parent, title = title,
													#size=(960, 390),
													style=wx.DEFAULT_FRAME_STYLE ^ (wx.RESIZE_BORDER |wx.MINIMIZE_BOX | wx.MAXIMIZE_BOX))#super the subclass
		#panel
		
		self.vboxsizer = wx.BoxSizer(wx.HORIZONTAL)
		
		self.inputP = inputPanel.InputPanel(self, -1, cSound=c)
		self.inputP.SetBackgroundColour((250, 250, 250))
		self.vboxsizer.Add(self.inputP, 0,wx.EXPAND)
		
		
		self.centralsizV = wx.BoxSizer(wx.VERTICAL)
		
		self.centralsizO = wx.BoxSizer(wx.HORIZONTAL)
		self.metronome = metroPanel.MetroPanel(self, -1, cSound=c, instr=15)
		self.metronome.SetBackgroundColour((200, 200, 200))
		self.centralsizO.Add(self.metronome, -1,wx.EXPAND)
		
		self.seqP1 = seqPanel.SeqPanel(self, -1, cSound=c, instr=1, basef_val=123.75)
		self.seqP1.SetBackgroundColour((200, 250, 200))
		self.centralsizO.Add(self.seqP1, -1,wx.EXPAND)
		self.seqP2 = seqPanel.SeqPanel(self, -1, cSound=c, instr=2, basef_val=185.625)
		self.seqP2.SetBackgroundColour((110, 250, 110))	
		self.centralsizO.Add(self.seqP2, -1,wx.EXPAND)
		self.seqP3 = seqPanel.SeqPanel(self, -1, cSound=c, instr=3, basef_val=278.437)
		self.seqP3.SetBackgroundColour((200, 250, 200))	
		self.centralsizO.Add(self.seqP3, -1,wx.EXPAND)
		#self.seqP4 = seqPanel.SeqPanel(self, -1, cSound=c, instr=4, basef_val=417.656)
		#self.seqP4.SetBackgroundColour((110, 250, 110))	
		#self.centralsizO.Add(self.seqP4, -1,wx.EXPAND)
		self.centralsizV.Add(self.centralsizO, 0,wx.EXPAND)
		
		self.matrixSeqP = dlyPanel.DlyPanel(self, -1, (-1,-1), cSound=c)
		self.matrixSeqP.SetBackgroundColour((100, 150, 100))
		self.centralsizV.Add(self.matrixSeqP, 0,wx.EXPAND)
		
		self.vboxsizer.Add(self.centralsizV, 0,wx.EXPAND)
		
		self.outputP = outputPanel.OutPanel(self, -1, cSound=c)
		self.vboxsizer.Add(self.outputP, 0,wx.EXPAND)
		
		#set sizer to panel
		#self.SetSizeHints(800, 300)
		#self.SetSize(wx.Size(800, 300))
		self.SetSizer(self.vboxsizer)
		self.vboxsizer.Fit(self)
		self.Show()

	def OnClose(self, event):
		"""to stop all the timers created"""
		self.inputP.timerRefresh.Stop() 
		self.seqP1.timerRefresh.Stop() 
		self.seqP2.timerRefresh.Stop() 
		self.seqP3.timerRefresh.Stop() 
		self.seqP4.timerRefresh.Stop() 
		self.seqP5.timerRefresh.Stop() 
		self.seqP6.timerRefresh.Stop() 
		self.outputP.timerRefresh.Stop() 
		self.Destroy()

class AppWithTerm(wx.App):
	"""wx.App subclassed to include csound Thread termination"""
	def OnExit(self):
		print "Closing csnd Thread"
		perfThread.Stop()
		print "Stopped csnd Thread"
		perfThread.Join()
		print "Closed csnd Thread"
		return


app = AppWithTerm(False)#True to redirect stdin/sterr
frame = TopFrame(None, title='Spengger')
#frame.Show()
app.MainLoop()
