import wx
import fsm
import wx.lib.agw.floatspin
import gridPanel

class DlyPanel(wx.Panel):
	def __init__(self, *a, **k):
		self.cSound = k.pop('cSound', None)
		super(DlyPanel, self).__init__(*a, **k)#super the subclass
		
		borderSizer = wx.BoxSizer(wx.HORIZONTAL)
		mainSizer = wx.BoxSizer(wx.VERTICAL)
		controlSizer = wx.GridBagSizer(vgap=3, hgap=3)
		self.dcw, self.dch = 600, 50
		self.buttw = 30
		siz=wx.Size(70,-1)#size of floatspin
		
		self.recordT = wx.ToggleButton(self, -1, label='Rec Time')
		self.Bind(wx.EVT_TOGGLEBUTTON, self.recordTime, self.recordT)
		
		delaysmaxT = wx.StaticText(self, -1, "Delays", style= wx.ALIGN_LEFT)
		self.delaysmaxV= fsm.FsmCs(parent=self, id=-1,
																digits=0,
																min_val = 1.0,
																max_val = 16.0,
																increment=1.0,
																value = 8.0,
																size = siz,
																cSound = self.cSound,
																channel = "kmaxDly")
		self.cSound.SetChannel("kmaxDly", 8)

		
		self.stepsDict = {"1":1.0,
									"1/8":0.5,
									"1/8T":0.5 * 2/3,
									"1/16":0.25,
									"1/16T":0.25 * 2/3,
									"1/32":0.125,
									"1/32T":0.125 * 2/3,
									"1/64":0.0625}
		self.stepsIndex = "1/8"
		self.stepsMultiplier = self.stepsDict[self.stepsIndex]
		self.stepsNames = self.stepsDict.keys()
		self.Stepvalue = wx.ComboBox(self, -1, choices=self.stepsNames, size=wx.Size(60, 20))
		self.Bind(wx.EVT_COMBOBOX, self.setStepDur, self.Stepvalue)
		self.Stepvalue.SetValue(self.stepsIndex)
		self.cSound.SetChannel("kduration", self.stepsDict[self.stepsNames[self.Stepvalue.GetSelection()]])
		
		stepsT = wx.StaticText(self, -1, "Steps", style= wx.ALIGN_LEFT)
		self.stepsV= fsm.FsmCs(parent=self, id=-1,
																			digits=0,
																			min_val = 1.0,
																			max_val = 64.0,
																			increment=1.0,
																			value = 16.0,
																			size = siz,
																			cSound = self.cSound,
																			channel = "ksteps")
		self.cSound.SetChannel("ksteps", 16)
		
		stepValueT = wx.StaticText(self, -1, "Time\nDiv", style = wx.ALIGN_LEFT)
		
		timeqtT = wx.StaticText(self, -1, "Time\nQuant", style= wx.ALIGN_LEFT)
		self.timeQt = wx.CheckBox(self, -1, style=wx.CHK_2STATE)
		self.Bind(wx.EVT_CHECKBOX, self.timeQuantizeSet, self.timeQt)
		
		
		self.resetco = wx.Button(self, -1, label='Reset Point')
		self.Bind(wx.EVT_BUTTON, self.resetCounter, self.resetco)
		
		self.clearmemory = wx.Button(self, -1, label='Reset Memory')
		self.Bind(wx.EVT_BUTTON, self.clearMemory, self.clearmemory)
		
		
		#Timer Update MUST BE STOPPED ON CLOSE
		self.timerRefresh = wx.Timer(self, wx.ID_ANY)
		self.timerRefresh.Start(100)
		self.Bind(wx.EVT_TIMER, self.timerUpdate, self.timerRefresh)
		
		
		#SIZER
		bord = 12
		controlSizer.Add(self.recordT, pos=(0,0))
		controlSizer.Add(delaysmaxT, pos=(0,1))
		controlSizer.Add(self.delaysmaxV, pos=(0,2))
		controlSizer.Add(stepValueT, pos=(0,3))
		controlSizer.Add(self.Stepvalue, pos=(0,4))
		controlSizer.Add(stepsT, pos=(0,5))
		controlSizer.Add(self.stepsV, pos=(0,6))
		controlSizer.Add(timeqtT, pos=(0,7))
		controlSizer.Add(self.timeQt, pos=(0,8))
		controlSizer.Add(self.resetco, pos=(0,9))
		controlSizer.Add(self.clearmemory, pos=(0,10))
		
		#Dly DC panel
		sizepanel = self.GetSize()
		self.gridPan = gridPanel.GridPanel(self, -1, dcw = self.dcw, dch = self.dch, stepsDict = self.stepsDict,  cSound=self.cSound)
		self.gridPan.SetSize(wx.Size(self.dcw, self.dch))
		#Display
		mainSizer.AddSpacer(5)
		mainSizer.Add(self.gridPan, 0)
		mainSizer.Add(controlSizer, 0, flag=wx.EXPAND)
		
		borderSizer.AddSpacer(5)
		borderSizer.Add(mainSizer, flag=wx.EXPAND)
		borderSizer.AddSpacer(5)
		#mainSizer.Add(self.outPanel,-1,flag=wx.EXPAND)
		self.SetSizer(borderSizer)
		borderSizer.Fit(self)


	def onClose(self, evt):
		self.timerRefresh.Stop() 
		self.Destroy()
		
		
	def recordTime(self, evt):
		'''set record flag in csd'''
		obj = evt.GetEventObject()
		ispressed = obj.GetValue()
		if ispressed:
			self.gridPan.recT = 1
			self.cSound.SetChannel("krecordDly", 1)
		else:
			self.gridPan.recT = 0
			self.cSound.SetChannel("krecordDly", 0)


	def setSteps(self, evt):
		'''set the steps of grid'''
		#self.gridPan.steps = self.stepsV.GetValue()
		#print self.stepsV.GetValue()
		pass

	def setStepDur(self, evt):
		"""set the step duration"""
		#self.gridPan.stepsIndex = self.stepsNames[self.Stepvalue.GetSelection()]
		dur = self.stepsDict[self.stepsNames[self.Stepvalue.GetSelection()]]
		self.gridPan.ksteps_dur = dur
		self.cSound.SetChannel("kduration", dur)
	
	def resetCounter(self, evt):
		self.cSound.InputMessage('i -25 0 -1\ni 25 0.001 -1\n')

	def clearMemory(self, evt):
		'''clear the delay memory'''
		self.cSound.InputMessage('i 28 0 -1')


	def timerUpdate(self, evt):
		"""timed actions"""
		self.gridPan.Refresh(False)#calling refresh is equal to wx.EVT_PAINT event

	def timeQuantizeSet(self, evt):
		"""time quantization checkbox managing"""
		#self.gridPan.timeQuantize = self.timeQt.GetValue()
		self.cSound.SetChannel("kquant",  self.timeQt.GetValue())

