import wx
from math import modf
import floatops
#import delayGUI

class GridPanel(wx.Panel):
	def __init__(self, *a, **k):
		self.dcw =  k.pop('dcw', None)
		self.dch = k.pop('dch', None)
		self.stepsDict = k.pop('stepsDict', None)
		self.cSound = k.pop('cSound', None)
		super(GridPanel, self).__init__(*a, **k)#super the subclass
		#musical data
		self.kpointer = 0.0#updated with callback
		self.ksteps_out = 8#updated with callback
		self.ksteps_dur_out = 0.5#updated with callback
		#sizer
		mainSizer = wx.BoxSizer(wx.HORIZONTAL)
		self.SetBackgroundColour((0, 100, 250))
		#recordingflag
		self.recT = 0
		#Quantize flag
		self.timeQt = 1
		#points storage
		self.points = []
		#Grid
		self.dcx, self.dcy = 0, 0
		self.bordx, self.bordy = 10, 10
		self.InitBuffer()
		self.Bind(wx.EVT_PAINT, self.OnPaint)#paint event
		
	
	
	
	
	def drawPointer(self, dc):
		'''time pointer'''
		if self.recT == 0:
			dc.SetPen(wx.Pen('GREY', 5, wx.SOLID))
		else:
			dc.SetPen(wx.Pen('RED', 5, wx.SOLID))
		if 0.0 > self.kpointer > 1.0: self.kpointer = 1.0#to avoid eventual inf in channel
		opos = self.bordx + (self.dcw - 2 * self.bordx) * self.kpointer
		dc.DrawLine(int(opos), self.bordy, int(opos), self.dch - self.bordy)
		
		
	def OnPaint(self, evt):
		dc = wx.BufferedPaintDC(self, self.buffer)
		self.DrawIt(dc)

	
	def InitBuffer(self):
		"""initialize drawing buffer"""
		self.buffer = wx.EmptyBitmap(self.dcw, self.dch)
		dc = wx.BufferedDC(None, self.buffer)
		dc.SetBackgroundMode(wx.TRANSPARENT)
		self.DrawIt(dc)
		

	def DrawIt(self, dc):
		"""Drawing procedure"""
		dc.SetDeviceOrigin(self.dcx, self.dcy)
		dc.SetBackgroundMode(wx.TRANSPARENT)
		dc.Clear()
		self.DrawAxis(dc)
		self.DrawPoints(dc)
		self.drawPointer(dc)
		
		
	def DrawAxis(self, dc):
		"""draw axis and indications on screen"""
		dc.SetPen(wx.Pen((200,200,200), 1, wx.SOLID))
		#dc.SetFont(wx.Font(8, wx.FONTFAMILY_DEFAULT, wx.NORMAL, wx.FONTWEIGHT_NORMAL))
		#Time Drawing
		dc.SetTextForeground((0,255,0))
		if self.ksteps_out <= 0: self.ksteps_out = 8#to avoid division by 0
		i = 0
		while i < self.ksteps_out + 1:
			opos = self.bordx + (self.dcw - 2 * self.bordx) * i / int(self.ksteps_out)
			dc.DrawLine(opos, self.bordy, opos, self.dch - self.bordy)
			i += 1
		#horizontal line
		dc.DrawLine(self.bordx, self.dch / 2, self.dcw - self.bordx, self.dch / 2)


	def DrawPoints(self, dc):
		"""draw the points on the screen"""
		dc.SetPen(wx.Pen('DARK GREEN', 2, wx.SOLID))
		c = 0
		kmax = self.ksteps_out * self.ksteps_dur_out
		while c < 16:
			val = self.cSound.TableGet(10, c)
			if val >= 0:
				opos = self.bordx + int((self.dcw - 2 * self.bordx) * val / kmax)
				dc.DrawLine(opos, self.bordy, opos, self.dch - self.bordy)
			c += 1
			




