import wx
import fsm

class InputPanel(wx.Panel):
	def __init__(self, *a, **k):
		self.cSound = k.pop('cSound', None)
		self.cSound_perf = k.pop('cSound_perf', None)
		super(InputPanel, self).__init__(*a, **k)#super the subclass
		#self.SetBackgroundColour((200, 200, 200))
		siz = wx.Size(70,-1)#size of floatspin
		
		#VUMETER
		self.vuSizer  = wx.GridBagSizer(vgap=10, hgap=10)
		vuT = wx.StaticText(self, -1, "dB", style= wx.ALIGN_CENTER | wx.TE_RICH)
		self.vumeter_in = wx.TextCtrl(self, -1, size=(60,20))
		
		#Gain
		gainT = wx.StaticText(self, -1, "Gain", style= wx.ALIGN_LEFT)
		self.gain = fsm.FsmCs(parent=self, id=-1,
				digits=3,
				min_val = 0.0,
				max_val = 3.0,
				increment=0.001,
				value = 1.0,
				size = siz,
				cSound = self.cSound,
				channel = "inGain")
		self.cSound.SetChannel("inGain", 1.0)

		
		#Trigger Threshold
		thresT = wx.StaticText(self, -1, "Trigger Threshold", style= wx.ALIGN_LEFT)
		self.thres =  fsm.FsmCs(parent=self, id=-1,
				digits=3,
				min_val = -150.0,
				max_val = 0.0,
				increment=0.001,
				value = -30.0,
				size = siz,
				cSound = self.cSound,
				channel = "inThresh")
		self.cSound.SetChannel("inThresh", -30.0)		
		
		#Modulator speed
		centrspeedT = wx.StaticText(self, -1, "Modulator Speed", style= wx.ALIGN_LEFT)
		self.centrspeed =  fsm.FsmCs(parent=self, id=-1,
				digits=3,
				min_val = 0.0,
				max_val = 30.0,
				increment=0.001,
				value = 0.0,
				size = siz,
				cSound = self.cSound,
				channel = "modulatorSpeed")
		
		shimdurT = wx.StaticText(self, -1, "Shimmering", style= wx.ALIGN_LEFT)
		self.shimdur = fsm.FsmCs(parent=self, id=-1,
				digits=3,
				min_val = 0.0,
				max_val = 1.0,
				increment=0.001,
				value = 0.5,
				size = siz,
				cSound = self.cSound,
				channel = "shimmerDur")
		self.cSound.SetChannel("shimmerDur", 0.5)
		
		shimHPT = wx.StaticText(self, -1, "ShimmerHP", style= wx.ALIGN_LEFT)
		self.shimHP = fsm.FsmCs(parent=self, id=-1,
				digits=1,
				min_val = 20.0,
				max_val = 5000.0,
				increment=0.1,
				value = 200.0,
				size = siz,
				cSound = self.cSound,
				channel = "shimmerHP")
		self.cSound.SetChannel("shimmerHP", 200.0)
		
		self.lockTempo = wx.ToggleButton(self, -1, label='Unlocked')
		self.Bind(wx.EVT_TOGGLEBUTTON, self.doLockBpM, self.lockTempo)#Send tap tempo events
		
		self.bpm = fsm.FsmCs(parent=self, id=-1,
				digits=2,
				min_val = 1.0,
				max_val = 500.0,
				increment=0.1,
				value = 60.0,
				size = siz,
				cSound = self.cSound,
				channel = "bpmIn")
		self.cSound.SetChannel("bpmIn", 60.0)
		
		bpmerrorT = wx.StaticText(self, -1, "Time Error", style= wx.ALIGN_LEFT)
		self.bpmerror = fsm.FsmCs(parent=self, id=-1,
				digits=1,
				min_val = 0.0,
				max_val = 100.0,
				increment=0.1,
				value = 5.0,
				size = siz,
				cSound = self.cSound,
				channel = "bpmError")
		self.cSound.SetChannel("bpmError", 5.0)
		
		
		self.cleanSeq = wx.Button(self, -1, label='Clean Seq')
		self.Bind(wx.EVT_BUTTON, self.doCleanSeq, self.cleanSeq)#Send tap tempo events
		
		#recording Button
		self.recB = wx.ToggleButton(self, -1, label='Record')
		self.Bind(wx.EVT_TOGGLEBUTTON, self.doRecord, self.recB)#Send tap tempo events
		
		
		#Timer Update MUST BE STOPPED IN MAIN FRAME
		self.timerRefresh = wx.Timer(self, wx.ID_ANY)
		self.timerRefresh.Start(50)
		self.Bind(wx.EVT_TIMER, self.timerUpdate, self.timerRefresh)
		
		
		
		#Sizer
		self.vuSizer.Add(vuT, pos=(0,0))
		self.vuSizer.Add(self.vumeter_in, pos=(0,1))
		self.vuSizer.Add(gainT, pos=(1,0))
		self.vuSizer.Add(self.gain, pos=(1,1))
		self.vuSizer.Add(thresT, pos=(2,0))
		self.vuSizer.Add(self.thres, pos=(2,1))
		self.vuSizer.Add(self.lockTempo, pos=(3,0))
		self.vuSizer.Add(self.bpm, pos=(3,1))
		self.vuSizer.Add(self.cleanSeq, pos=(4,0))
		self.vuSizer.Add(bpmerrorT, pos=(5,0))
		self.vuSizer.Add(self.bpmerror, pos=(5,1))
		
		self.vuSizer.Add(centrspeedT, pos=(6,0))
		self.vuSizer.Add(self.centrspeed, pos=(6,1))
		self.vuSizer.Add(shimdurT, pos=(7,0))
		self.vuSizer.Add(self.shimdur, pos=(7,1))
		self.vuSizer.Add(shimHPT, pos=(8,0))
		self.vuSizer.Add(self.shimHP, pos=(8,1))
		
		
		self.vuSizer.Add(self.recB, pos=(10,0))
		
		
		
		#self.SetSizer(self.vuSizer)
		#self.vuSizer.Fit(self)
		#to add border to the panel
		sizx = wx.BoxSizer(wx.HORIZONTAL)
		sizy = wx.BoxSizer(wx.VERTICAL)
		sizx.AddSpacer(5,wx.EXPAND)
		sizx.Add(self.vuSizer, 0,wx.EXPAND)
		sizx.AddSpacer(5,wx.EXPAND)
		sizy.AddSpacer(5,wx.EXPAND)
		sizy.Add(sizx, 0,wx.EXPAND)
		sizy.AddSpacer(5,wx.EXPAND)
		self.SetSizer(sizy)
		sizy.Fit(self)



	
	def timerUpdate(self, evt):
		"""update the vumeters"""
		self.db = self.cSound.GetChannel("inVUmeter")
		if self.db < -1.0:
			self.vumeter_in.SetForegroundColour((100,250, 100))
		elif -1.0<= self.db < 0.0:
			self.vumeter_in.SetForegroundColour((250, 200, 0))
		else:
			self.vumeter_in.SetForegroundColour((250, 10, 10))
		self.vumeter_in.SetValue(str(self.db))
		"""update BpM if time is not locked"""
		if not self.lockTempo.GetValue():
			bpmCs = self.cSound.GetChannel("bpmOut")
			self.bpm.SetValue(bpmCs)
		
		
	def doLockBpM(self, evt):
		"""void the Bpm Change instrument"""
		obj = evt.GetEventObject()
		ispressed = obj.GetValue()
		if ispressed:
			obj.SetLabel('Locked->')
			self.cSound.SetChannel("lockBpM", 1)
		else:
			obj.SetLabel('Unlocked')
			self.cSound.SetChannel("lockBpM", 0)
		
	def doCleanSeq(self, evt):
		"""activate cleaning of the sequencer table"""
		self.cSound.InputMessage('i 4 0 -1\n')
		
	def doRecord(self, evt):
		"""lauch record instrument"""
		obj = evt.GetEventObject()
		ispressed = obj.GetValue()
		if ispressed:
			obj.SetLabel('Recording')
			self.cSound.InputMessage('i 101 0 -1\n')
		else:
			obj.SetLabel('Stopped')
			self.cSound.InputMessage('i -101 0 -1\n')
