import wx
import fsm

class MetroPanel(wx.Panel):
	def __init__(self, *a, **k):
		self.cSound = k.pop('cSound', None)
		self.instr = k.pop('instr', None)
		super(MetroPanel, self).__init__(*a, **k)#super the subclass
		#self.SetBackgroundColour((250, 200, 200))
		siz = wx.Size(70,-1)#size of floatspin
		self.sizer = wx.BoxSizer(wx.VERTICAL)
		
		self.PlayB = wx.ToggleButton(self, -1, label='Metro OFF')
		self.Bind(wx.EVT_TOGGLEBUTTON, self.doPlay, self.PlayB)#Send tap tempo events
		
		self.stepSizer  = wx.GridBagSizer(vgap=10, hgap=10)
		
		speedT = wx.StaticText(self, -1, "Speed", style= wx.ALIGN_CENTER | wx.TE_RICH)
		self.speedList = ["0.125", "0.25", "0.5", "0.75", "1.0", "2.0", "3.0", "4.0", "5.0", "6.0", "7.0", "8.0"] 
		self.speedvalue = wx.ComboBox(self, -1, choices=self.speedList, size=siz)
		self.speedvalue.SetValue("1.0")
		self.Bind(wx.EVT_COMBOBOX, self.setSpeed, self.speedvalue)
		self.cSound.SetChannel("metroSpeed", 1.0)
		
		
		accentT = wx.StaticText(self, -1, "accent", style= wx.ALIGN_CENTER | wx.TE_RICH)
		self.accent = fsm.FsmCs(parent=self, id=-1,
																			digits=0,
																			min_val = 1.0,
																			max_val = 128.0,
																			increment=1.0,
																			value = 4,
																			size = siz,
																			cSound = self.cSound,
																			channel = "accent")
		self.cSound.SetChannel("accent", 4)
		
		timingT = wx.StaticText(self, -1, "timing", style= wx.ALIGN_CENTER | wx.TE_RICH)
		self.timing = fsm.FsmCs(parent=self, id=-1,
																			digits=3,
																			min_val = 0.0,
																			max_val = 1.0,
																			increment=0.001,
																			value = 0.0,
																			size = siz,
																			cSound = self.cSound,
																			channel = "timing")
		self.cSound.SetChannel("timing", 0.0)
		
		randvolT = wx.StaticText(self, -1, "Rnd Vol", style= wx.ALIGN_CENTER | wx.TE_RICH)
		self.randvol = fsm.FsmCs(parent=self, id=-1,
																			digits=2,
																			min_val = 0.0,
																			max_val = 1.0,
																			increment=0.01,
																			value = 0.5,
																			size = siz,
																			cSound = self.cSound,
																			channel = "randvol")
		self.cSound.SetChannel("randvol", 0.5)
		
		excludeT = wx.StaticText(self, -1, "exclude", style= wx.ALIGN_CENTER | wx.TE_RICH)
		self.exclude = fsm.FsmCs(parent=self, id=-1,
																			digits=2,
																			min_val = 0.0,
																			max_val = 1.0,
																			increment=0.01,
																			value = 0.5,
																			size = siz,
																			cSound = self.cSound,
																			channel = "exclude")
		
		doubleT = wx.StaticText(self, -1, "double\nstroke", style= wx.ALIGN_CENTER | wx.TE_RICH)
		self.double = fsm.FsmCs(parent=self, id=-1,
																			digits=2,
																			min_val = 0.0,
																			max_val = 1.0,
																			increment=0.01,
																			value = 0.3,
																			size = siz,
																			cSound = self.cSound,
																			channel = "double")
		self.cSound.SetChannel("double", 0.3)
		
		basefreT = wx.StaticText(self, -1, "base\nfreq", style= wx.ALIGN_CENTER | wx.TE_RICH)
		self.basefre = fsm.FsmCs(parent=self, id=-1,
																			digits=1,
																			min_val = 20.0,
																			max_val = 880.0,
																			increment=0.50,
																			value = 210.0,
																			size = siz,
																			cSound = self.cSound,
																			channel = "kmetrofreq")
		self.cSound.SetChannel("kmetrofreq", 210.0)
		
		self.stepSizer.Add(speedT, pos=(0,0))
		self.stepSizer.Add(self.speedvalue, pos=(0,1))
		
		self.stepSizer.Add(accentT, pos=(1,0))
		self.stepSizer.Add(self.accent, pos=(1,1))
		
		self.stepSizer.Add(timingT, pos=(2,0))
		self.stepSizer.Add(self.timing, pos=(2,1))
		
		self.stepSizer.Add(randvolT, pos=(3,0))
		self.stepSizer.Add(self.randvol, pos=(3,1))
		
		self.stepSizer.Add(excludeT, pos=(4,0))
		self.stepSizer.Add(self.exclude, pos=(4,1))
		
		self.stepSizer.Add(doubleT, pos=(5,0))
		self.stepSizer.Add(self.double, pos=(5,1))
		
		self.stepSizer.Add(basefreT, pos=(6,0))
		self.stepSizer.Add(self.basefre, pos=(6,1))
		
		self.sizer.Add(self.PlayB, 0,flag=wx.EXPAND)
		self.sizer.AddSpacer(10,flag=wx.EXPAND)
		self.sizer.Add(self.stepSizer, 0,flag=wx.EXPAND)
		
		#to add border to the panel
		sizx = wx.BoxSizer(wx.HORIZONTAL)
		sizy = wx.BoxSizer(wx.VERTICAL)
		sizx.AddSpacer(5,wx.EXPAND)
		sizx.Add(self.sizer, 0,wx.EXPAND)
		sizx.AddSpacer(5,wx.EXPAND)
		sizy.AddSpacer(5,wx.EXPAND)
		sizy.Add(sizx, 0,wx.EXPAND)
		sizy.AddSpacer(5,wx.EXPAND)
		self.SetSizer(sizy)
		sizy.Fit(self)
		
		
		

	def doPlay(self, evt):
		"""send play activation value"""
		obj = evt.GetEventObject()
		ispressed = obj.GetValue()
		if ispressed:
			#self.cSound.InputMessage('i %d 0 -1\n' % self.instr)
			obj.SetLabel('Metro Running')
			self.cSound.InputMessage('i %f 0 -1\n' % 15)
		else:
			#self.cSound.InputMessage('i %d 0 -1\n' % (-1 * self.instr))
			obj.SetLabel('Metro OFF')
			self.cSound.InputMessage('i %f 0 -1\n' % -15)
		
	
	def setSpeed(self, evt):
		"""set the speed multiplier"""
		speed = float(self.speedList[self.speedvalue.GetSelection()])
		self.cSound.SetChannel("metroSpeed", speed)
		
		