import wx
import fsm

class SeqPanel(wx.Panel):
	def __init__(self, *a, **k):
		self.cSound = k.pop('cSound', None)
		self.cSound_perf = k.pop('cSound_perf', None)
		self.instr = k.pop('instr', None)
		self.basef_val= k.pop('basef_val', None)
		super(SeqPanel, self).__init__(*a, **k)#super the subclass
		#self.SetBackgroundColour((250, 200, 200))
		siz = wx.Size(70,-1)#size of floatspin
		self.sizer = wx.BoxSizer(wx.VERTICAL)
		
		self.PlayB = wx.ToggleButton(self, -1, label='Stop_Seq_%d'% self.instr)
		self.Bind(wx.EVT_TOGGLEBUTTON, self.doPlay, self.PlayB)#Send tap tempo events
		#self.running = 0
		#self.PlayB = wx.Button(self, -1, label='Play')
		#self.Bind(wx.EVT_BUTTON, self.doPlay, self.PlayB)#Send tap tempo events
		
		self.stepSizer  = wx.GridBagSizer(vgap=10, hgap=10)
		stepNT = wx.StaticText(self, -1, "stepN", style= wx.ALIGN_CENTER | wx.TE_RICH)
		self.stepN = wx.TextCtrl(self, -1, size=siz)
		stepVT = wx.StaticText(self, -1, "StepV", style= wx.ALIGN_CENTER | wx.TE_RICH)
		self.stepV = wx.TextCtrl(self, -1, size=siz)
		
		speedT = wx.StaticText(self, -1, "Speed", style= wx.ALIGN_CENTER | wx.TE_RICH)
		self.speedList = ["0.125", "0.25", "0.5", "0.75", "1.0", "2.0", "3.0", "4.0", "5.0", "6.0", "7.0", "8.0"] 
		self.speedvalue = wx.ComboBox(self, -1, choices=self.speedList, size=siz)
		self.speedvalue.SetValue("1.0")
		self.Bind(wx.EVT_COMBOBOX, self.setSpeed, self.speedvalue)
		self.cSound.SetChannel("stepSpeed%d" % self.instr, 1.0)
		
		maxstepT = wx.StaticText(self, -1, "Steps", style= wx.ALIGN_CENTER | wx.TE_RICH)
		self.maxstep = fsm.FsmCs(parent=self, id=-1,
																			digits=0,
																			min_val = 1.0,
																			max_val = 128.0,
																			increment=1.0,
																			value = 32,
																			size = siz,
																			cSound = self.cSound,
																			channel = "maxStep%d" % self.instr)
		self.cSound.SetChannel("maxStep%d" % self.instr, 32)
		
		beatenT = wx.StaticText(self, -1, "BEATER", style= wx.ALIGN_CENTER | wx.TE_RICH)
		
		duratT = wx.StaticText(self, -1, "duration", style= wx.ALIGN_CENTER | wx.TE_RICH)
		self.durat = fsm.FsmCs(parent=self, id=-1,
																			digits=3,
																			min_val = 0.1,
																			max_val = 1.5,
																			increment=0.01,
																			value = 0.5,
																			size = siz,
																			cSound = self.cSound,
																			channel = "durat%d" % self.instr)
		self.cSound.SetChannel("durat%d" % self.instr, 0.5)
		
		basefT = wx.StaticText(self, -1, "baseF", style= wx.ALIGN_CENTER | wx.TE_RICH)
		self.basef =  fsm.FsmCs(parent=self, id=-1,
																			digits=1,
																			min_val = 30.0,
																			max_val = 10000.0,
																			increment = 0.5,
																			value = self.basef_val,
																			size = siz,
																			cSound = self.cSound,
																			channel = "basef%d" % self.instr)
		self.cSound.SetChannel("basef%d" % self.instr, self.basef_val)
		
		ratiofT = wx.StaticText(self, -1, "ratio", style= wx.ALIGN_CENTER | wx.TE_RICH)
		self.ratiof = fsm.FsmCs(parent=self, id=-1,
																			digits=2,
																			min_val = 0.1,
																			max_val = 8.0,
																			increment = 0.01,
																			value = 3.91,
																			size = siz,
																			cSound = self.cSound,
																			channel = "ratiof%d" % self.instr)
		self.cSound.SetChannel("ratiof%d" % self.instr, 3.91)
		
		#Timer Update MUST BE STOPPED IN MAIN FRAME
		self.timerRefresh = wx.Timer(self, wx.ID_ANY)
		self.timerRefresh.Start(50)
		self.Bind(wx.EVT_TIMER, self.updateStepTimer, self.timerRefresh)		
		
		
		
		self.stepSizer.Add(stepNT, pos=(0,0))
		self.stepSizer.Add(self.stepN, pos=(0,1))
		self.stepSizer.Add(stepVT, pos=(1,0))
		self.stepSizer.Add(self.stepV, pos=(1,1))
		self.stepSizer.Add(speedT, pos=(2,0))
		self.stepSizer.Add(self.speedvalue, pos=(2,1))
		self.stepSizer.Add(maxstepT, pos=(3,0))
		self.stepSizer.Add(self.maxstep, pos=(3,1))
		
		self.stepSizer.Add(beatenT, pos=(4,1))
		self.stepSizer.Add(duratT, pos=(5,0))
		self.stepSizer.Add(self.durat, pos=(5,1))
		self.stepSizer.Add(basefT, pos=(6,0))
		self.stepSizer.Add(self.basef, pos=(6,1))
		self.stepSizer.Add(ratiofT, pos=(7,0))
		self.stepSizer.Add(self.ratiof, pos=(7,1))
		
		
		self.sizer.Add(self.PlayB, 0,flag=wx.EXPAND)
		self.sizer.AddSpacer(10,flag=wx.EXPAND)
		self.sizer.Add(self.stepSizer, 0,flag=wx.EXPAND)
		
		#self.SetSizer(self.sizer)
		#self.sizer.Fit(self)
		
		#to add border to the panel
		sizx = wx.BoxSizer(wx.HORIZONTAL)
		sizy = wx.BoxSizer(wx.VERTICAL)
		sizx.AddSpacer(5,wx.EXPAND)
		sizx.Add(self.sizer, 0,wx.EXPAND)
		sizx.AddSpacer(5,wx.EXPAND)
		sizy.AddSpacer(5,wx.EXPAND)
		sizy.Add(sizx, 0,wx.EXPAND)
		sizy.AddSpacer(5,wx.EXPAND)
		self.SetSizer(sizy)
		sizy.Fit(self)
		
		
		

	def doPlay(self, evt):
		"""send play activation value"""
		obj = evt.GetEventObject()
		ispressed = obj.GetValue()
		if ispressed:
			#self.cSound.InputMessage('i %d 0 -1\n' % self.instr)
			obj.SetLabel('Running Seq_%d'% self.instr)
			self.cSound.InputMessage('i %f 0 -1 %d\n' % (10 + 0.01 * self.instr, self.instr))
		else:
			#self.cSound.InputMessage('i %d 0 -1\n' % (-1 * self.instr))
			obj.SetLabel('Stopped Seq_%d'% self.instr)
			self.cSound.InputMessage('i %f 0 -1\n' % (-10 - 0.01 * self.instr))
		
	
	def updateStepTimer(self,evt):
		step_n = self.cSound.GetChannel("stepOutN%d" % self.instr)
		self.stepN.SetValue('%d' % step_n)
		step_v = self.cSound.GetChannel("stepOutV%d" % self.instr)
		self.stepV.SetValue('%d' % step_v)

	
	
	def setSpeed(self, evt):
		"""set the speed multiplier"""
		speed = float(self.speedList[self.speedvalue.GetSelection()])
		self.cSound.SetChannel("stepSpeed%d" % self.instr, speed)
		
		